@@ -183,7 +183,7 @@
 #define Debug18(m,a1,a2,a3,a4,a5,a6,a7,a8,a9,a10,a11,a12,a13,a14,a15,a16,a17,a18)
 #endif /* !(WITH_MSGLEVEL <= E_DEBUG) */
 
-/* message with software controlled serverity */
+/* message with software controlled severity */
 #if WITH_MSGLEVEL <= E_FATAL
 #define Msg(l,m) msg(l,"%s",m)
 #define Msg1(l,m,a1) msg(l,m,a1)
@@ -217,7 +217,7 @@ struct diag_dgram {
    enum diag_op op;
 #if HAVE_CLOCK_GETTIME
    struct timespec now;
-#elif HAVE_GETTIMEOFDAY
+#elif HAVE_PROTOTYPE_LIB_gettimeofday
    struct timeval now;
 #else
    time_t now;
@@ -228,14 +228,16 @@ struct diag_dgram {
    char text[TEXTLEN];
 } ;
 
-extern sig_atomic_t diag_in_handler;
-extern sig_atomic_t diag_immediate_msg;
-extern sig_atomic_t diag_immediate_exit;
+extern volatile sig_atomic_t diag_in_handler;
+extern volatile sig_atomic_t diag_immediate_msg;
+extern volatile sig_atomic_t diag_immediate_exit;
 
 extern void diag_set(char what, const char *arg);
 extern void diag_set_int(char what, int arg);
 extern int diag_get_int(char what);
 extern const char *diag_get_string(char what);
+extern int diag_reserve_fd(int fd);
+extern int diag_fork(void);
 extern int diag_dup(void);
 extern int diag_dup2(int newfd);
 extern void msg(int level, const char *format, ...);
