@@ -22,14 +22,27 @@ const int one = 1;
 
 /* Substitute for Write():
    Try to write all bytes before returning; this handles EINTR,
-   EAGAIN/EWOULDBLOCK, and partial write situations. The drawback is that this
-   function might block even with O_NONBLOCK option.
+   EAGAIN/EWOULDBLOCK, and partial write situations. The drawback of this
+   function is that it blocks even with O_NONBLOCK option.
    Returns <0 on unhandled error, errno valid
    Will only return <0 or bytes
 */
-ssize_t writefull(int fd, const void *buff, size_t bytes) {
+/* Assuming your pipe size is 65536 - find out with:
+   filan -i 1 |grep F_GETPIPE_SZ |sed 's|.*\(F_GETPIPE_SZ=[1-9][0-9]*\).*|\1|'
+   Then we can test partial write with something like:
+   socat -d4 -lu -b 262144 -u /dev/zero,readbytes=262144 -,o-nonblock |{ sleep 3; wc -c; }
+*/
+ssize_t writefull(
+	int fd,
+	const void *buff,
+	size_t bytes,
+	const struct timeval *tmo0) {
    size_t writt = 0;
    ssize_t chk;
+   struct pollfd pfd;
+   struct timeval tmo = { 0 };
+   int rc;
+
    while (1) {
       chk = Write(fd, (const char *)buff + writt, bytes - writt);
       if (chk < 0) {
@@ -40,15 +53,34 @@ ssize_t writefull(int fd, const void *buff, size_t bytes) {
 	 case EWOULDBLOCK:
 #endif
 	    Warn4("write(%d, %p, "F_Zu"): %s", fd, (const char *)buff+writt, bytes-writt, strerror(errno));
-	    Sleep(1); continue;
-	 default: return -1;
+	    pfd.fd      = fd;
+	    pfd.events  = POLLOUT;
+	    pfd.revents = 0;
+	    if (tmo0 != NULL) {
+	       tmo.tv_sec  = tmo0->tv_sec;
+	       tmo.tv_usec = tmo0->tv_usec;
+	    }
+	    rc = xiopoll(&pfd, 1, (tmo.tv_sec!=0 || tmo.tv_usec!=0) ? &tmo : NULL);
+	    if (rc == 0) {
+	       Notice("inactivity timeout triggered");
+	       errno = ETIMEDOUT;
+	       return -1;
+	    }
+	    continue;
+	 default:
+	    return -1;
 	 }
+      } else if (chk == bytes) {
+	 /* First attempt, complete write */
+	 return chk;
       } else if (writt+chk < bytes) {
-	 Warn4("write(%d, %p, "F_Zu"): only wrote "F_Zu" bytes, trying to continue (rev.direction is blocked)",
+	 Warn4("write(%d, %p, "F_Zu"): only wrote "F_Zu" bytes, trying to continue (meanwhile, other direction is blocked)",
 	       fd, (const char *)buff+writt, bytes-writt, chk);
 	 writt += chk;
-      } else {
-	 writt = bytes;
+      } else if (writt == 0) {
+	 /* First attempt, write complete - no extra message */
+	 return chk;
+      } else { 	/* write completed */
 	 break;
       }
    }
@@ -56,6 +88,59 @@ ssize_t writefull(int fd, const void *buff, size_t bytes) {
    return writt;
 }
 
+
+/* Substitute for Read():
+   Try to read as many bytes as specified before returning; this handles EINTR,
+   EAGAIN/EWOULDBLOCK, and insufficient bytes; on EOF it returns the data read
+   so far.
+   The drawback of this function is that it blocks even with O_NONBLOCK option.
+   Returns <0 and errno on unhandled error;
+   returns the requested number, or less when EOF occurred; in the latter case,
+   the caller will get EOF on the next call to readfull().
+*/
+ssize_t readfull(
+	int fd,
+	void *buff,
+	size_t bytes,
+	const struct timeval *tmo0,	/* currently ignored */
+	int level)
+{
+	ssize_t result;
+	size_t exptg;	/* still expecting so many bytes */
+
+	exptg = bytes;
+	while (exptg > 0) {
+		result = Read(fd, buff, exptg);
+		if (result < 0 &&
+		    (errno == EINTR || errno == EAGAIN || errno == EWOULDBLOCK))
+			continue;
+		if (result < 0) {
+			Msg4(level, "read(%d, %p, "F_Zu"): %s",
+			     fd, buff, exptg, strerror(errno));
+			if (Close(fd) < 0) {
+				Info2("close(%d): %s", fd, strerror(errno));
+			}
+			return -1;
+		}
+		if (result == 0) {
+			Msg2(level, "read(): EOF during read(), returning "F_Zu" instead of "F_Zu" bytes",
+			    bytes-exptg, bytes);
+			return bytes-exptg;
+		}
+
+		*(char **)&buff += result;
+		exptg -= result;
+		if (exptg <= 0) {
+			break;
+		}
+		Debug2("readfull(): received "F_Zd" bytes, waiting for "F_Zu" more bytes",
+			result, exptg);
+	}
+	Debug1("readfull(): received all "F_Zd" bytes", bytes);
+	return bytes-exptg;
+}
+
+
 #if WITH_UNIX
 void socket_un_init(struct sockaddr_un *sa) {
 #if HAVE_STRUCT_SOCKADDR_SALEN
@@ -324,7 +409,10 @@ int sockaddr_vm_parse(struct sockaddr_vm *sa, const char *cid_str,
                       const char *port_str)
 {
    char *garbage = NULL;
-   if (!cid_str) {
+   memset(sa, 0, sizeof(*sa));
+   sa->svm_family = AF_VSOCK;
+
+   if (!cid_str || !cid_str[0]) {
       sa->svm_cid = VMADDR_CID_ANY;
    } else {
       sa->svm_cid = strtoul(cid_str, &garbage, 0);
@@ -346,7 +434,7 @@ int sockaddr_vm_parse(struct sockaddr_vm *sa, const char *cid_str,
 
    return 0;
 }
-#endif /* WITH_IP4 */
+#endif /* WITH_VSOCK */
 
 #if !HAVE_INET_NTOP
 /* http://www.opengroup.org/onlinepubs/000095399/functions/inet_ntop.html */
@@ -548,7 +636,7 @@ int getusergroups(const char *user, gid_t *list, int *ngroups) {
    /* we prefer getgrouplist because it may be much faster with many groups, but it is not standard */
    gid_t grp, twogrps[2];
    int two = 2;
-   /* getgrouplist requires to pass an extra group id, typically the users primary group, that is then added to the supplementary group list. We don't want such an additional group in the result, but there is not "unspecified" gid value available. Thus we try to find an abitrary supplementary group id that we then pass in a second call to getgrouplist. */
+   /* getgrouplist requires to pass an extra group id, typically the users primary group, that is then added to the supplementary group list. We don't want such an additional group in the result, but there is not "unspecified" gid value available. Thus we try to find an arbitrary supplementary group id that we then pass in a second call to getgrouplist. */
    grp = 0;
    Getgrouplist(user, grp, twogrps, &two);
    if (two == 1) {
@@ -677,7 +765,7 @@ int xiopoll(struct pollfd fds[], unsigned long nfds, struct timeval *timeout) {
 }
 
 
-#if WITH_TCP || WITH_UDP
+#if WITH_TCP || WITH_UDP || WITH_SCTP || WITH_DCCP || WITH_UDPLITE
 /* returns port in network byte order;
    ipproto==IPPROTO_UDP resolves as UDP service, every other value resolves as
    TCP */
@@ -702,10 +790,10 @@ int parseport(const char *portname, int ipproto) {
 
    return se->s_port;
 }
-#endif /* WITH_TCP || WITH_UDP */
+#endif /* WITH_TCP || WITH_UDP || WITH_SCTP || WITH_DCCP || WITH_UDPLITE */
 
 
-#if WITH_IP4 || WITH_IP6 || WITH_INTERFACE
+#if WITH_IP4 || WITH_IP6 || _WITH_INTERFACE
 /* check the systems interfaces for ifname and return its index
    or -1 if no interface with this name was found
    The system calls require an arbitrary socket; the calling program may
@@ -729,8 +817,8 @@ int ifindexbyname(const char *ifname, int anysock) {
    }
    if (anysock >= 0) {
       s = anysock;
-   } else  if ((s = Socket(PF_INET, SOCK_DGRAM, IPPROTO_IP)) < 0) {
-      Error1("socket(PF_INET, SOCK_DGRAM, IPPROTO_IP): %s", strerror(errno));
+   } else  if ((s = Socket(PF_INET, SOCK_DGRAM, 0)) < 0) {
+      Error1("socket(PF_INET, SOCK_DGRAM, 0): %s", strerror(errno));
       return -1;
    }
 
@@ -756,10 +844,10 @@ int ifindexbyname(const char *ifname, int anysock) {
    return -1;
 #endif /* !defined(HAVE_ STRUCT_IFREQ) && defined(SIOCGIFCONF) && defined(SIOCGIFINDEX) */
 }
-#endif /* WITH_IP4 || WITH_IP6 || WITH_INTERFACE */
+#endif /* WITH_IP4 || WITH_IP6 || _WITH_INTERFACE */
 
 
-#if WITH_IP4 || WITH_IP6 || WITH_INTERFACE
+#if WITH_IP4 || WITH_IP6 || _WITH_INTERFACE
 /* like ifindexbyname(), but also allows the index number as input - in this
    case it does not lookup the index.
    writes the resulting index to *ifindex and returns 0,
@@ -783,7 +871,7 @@ int ifindex(const char *ifname, unsigned int *ifindex, int anysock) {
    *ifindex = val;
    return 0;
 }
-#endif /* WITH_IP4 || WITH_IP6 || WITH_INTERFACE */
+#endif /* WITH_IP4 || WITH_IP6 || _WITH_INTERFACE */
 
 
 int _xiosetenv(const char *envname, const char *value, int overwrite, const char *sep) {
