@@ -1,5 +1,5 @@
 /* source: xio-ip4.h */
-/* Copyright Gerhard Rieger 2001-2008 */
+/* Copyright Gerhard Rieger and contributors (see file CHANGES) */
 /* Published under the GNU General Public License V.2, see file COPYING */
 
 #ifndef __xio_ip4_h_included
@@ -7,7 +7,7 @@
 
 extern const struct optdesc opt_ip4_add_membership;
 
-int xioparsenetwork_ip4(const char *rangename, struct xiorange *range);
+extern int xioparsenetwork_ip4(const char *rangename, struct xiorange *range, const int ai_flags[2]);
 extern
 int xiocheckrange_ip4(struct sockaddr_in *pa, struct xiorange *range);
 extern int
