@@ -1,5 +1,5 @@
-/* $Id: xio-listen.h,v 1.10 2006/07/13 06:44:53 gerhard Exp $ */
-/* Copyright Gerhard Rieger 2001-2006 */
+/* source: xio-listen.h */
+/* Copyright Gerhard Rieger and contributors (see file CHANGES) */
 /* Published under the GNU General Public License V.2, see file COPYING */
 
 #ifndef __xio_listen_h_included
@@ -7,7 +7,10 @@
 
 extern const struct optdesc opt_backlog;
 extern const struct optdesc opt_fork;
+extern const struct optdesc opt_max_children;
+extern const struct optdesc opt_children_shutup;
 extern const struct optdesc opt_range;
+extern const struct optdesc opt_accept_timeout;
 
 int
    xioopen_listen(struct single *xfd, int xioflags,
@@ -17,5 +20,6 @@ int
 int _xioopen_listen(struct single *fd, int xioflags,
 		    struct sockaddr *us, socklen_t uslen,
 		 struct opt *opts, int pf, int socktype, int proto, int level);
+extern int _xioopen_accept_fd(struct single *xfd, int xioflags, struct sockaddr *us, socklen_t uslen, struct opt *opts, int pf, int proto, int level);
 
 #endif /* !defined(__xio_listen_h_included) */
