@@ -25,29 +25,32 @@ uses stdin!!
 /* length of buffer for dynamic prompt */
 #define READLINE_MAXPROMPT 512
 
-static int xioopen_readline(int argc, const char *argv[], struct opt *opts,
-			    int rw, xiofile_t *xfd, unsigned groups,
-			    int dummy1, int dummy2, int dummy3);
+static int xioopen_readline(int argc, const char *argv[], struct opt *opts, int rw, xiofile_t *xfd, const struct addrdesc *addrdesc);
 
 
-const struct addrdesc addr_readline = {
-   "readline", 3, xioopen_readline, GROUP_FD|GROUP_TERMIOS|GROUP_READLINE, 0, 0, 0 HELP(NULL) };
+const struct addrdesc xioaddr_readline = { "READLINE", 3, xioopen_readline, GROUP_FD|GROUP_TERMIOS|GROUP_READLINE, 0, 0, 0 HELP(NULL) };
 
 const struct optdesc opt_history_file = { "history-file", "history", OPT_HISTORY_FILE, GROUP_READLINE, PH_LATE, TYPE_STRING, OFUNC_OFFSET, XIO_OFFSETOF(para.readline.history_file) };
 const struct optdesc opt_prompt       = { "prompt",       NULL,      OPT_PROMPT,       GROUP_READLINE, PH_LATE, TYPE_STRING, OFUNC_OFFSET, XIO_OFFSETOF(para.readline.prompt) };
 const struct optdesc opt_noprompt     = { "noprompt",     NULL,      OPT_NOPROMPT,     GROUP_READLINE, PH_LATE, TYPE_BOOL,   OFUNC_SPEC,   0 };
 const struct optdesc opt_noecho       = { "noecho",       NULL,      OPT_NOECHO,       GROUP_READLINE, PH_LATE, TYPE_STRING, OFUNC_SPEC,   0 };
 
-static int xioopen_readline(int argc, const char *argv[], struct opt *opts,
-			    int xioflags, xiofile_t *xfd, unsigned groups,
-			    int dummy1, int dummy2, int dummy3) {
+static int xioopen_readline(
+	int argc,
+	const char *argv[],
+	struct opt *opts,
+	int xioflags,
+	xiofile_t *xfd,
+	const struct addrdesc *addrdesc)
+{
+   struct single *sfd = &xfd->stream;
    int rw = (xioflags & XIO_ACCMODE);
    char msgbuf[256], *cp = msgbuf;
    bool noprompt = false;
    char *noecho = NULL;
 
    if (argc != 1) {
-      Error1("%s: 0 parameters required", argv[0]);
+      xio_syntax(argv[0], 0, argc-1, addrdesc->syntax);
       return STAT_NORETRY;
    }
 
@@ -72,7 +75,8 @@ static int xioopen_readline(int argc, const char *argv[], struct opt *opts,
    Notice(msgbuf);
 
    xfd->stream.fd = 0;	/* stdin */
-   xfd->stream.howtoend = END_NONE;
+   if (sfd->howtoend == END_UNSPEC)
+      xfd->stream.howtoend = END_NONE;
    xfd->stream.dtype    = XIODATA_READLINE;
 
 #if WITH_TERMIOS
@@ -86,10 +90,10 @@ static int xioopen_readline(int argc, const char *argv[], struct opt *opts,
    }
 #endif /* WITH_TERMIOS */
 
-   if (applyopts_single(&xfd->stream, opts, PH_INIT) < 0)  return -1;
-   applyopts(-1, opts, PH_INIT);
+   if (applyopts_single(sfd, opts, PH_INIT) < 0)  return -1;
+   applyopts(sfd, -1, opts, PH_INIT);
 
-   applyopts2(xfd->stream.fd, opts, PH_INIT, PH_FD);
+   applyopts2(sfd, -1, opts, PH_INIT, PH_FD);
 
    Using_history();
    applyopts_offset(&xfd->stream, opts);
@@ -123,8 +127,8 @@ static int xioopen_readline(int argc, const char *argv[], struct opt *opts,
       Read_history(xfd->stream.para.readline.history_file);
    }
 #if _WITH_TERMIOS
-   xiotermios_clrflag(xfd->stream.fd, 3, ICANON);
-   xiotermios_clrflag(xfd->stream.fd, 3, ECHO);
+   xiotermios_clrflag(xfd->stream.fd, 3, ICANON|ECHO);
+   xiotermios_flush(xfd->stream.fd);
 #endif /* _WITH_TERMIOS */
    return _xio_openlate(&xfd->stream, opts);
 }
@@ -174,12 +178,13 @@ ssize_t xioread_readline(struct single *pipe, void *buff, size_t bufsiz) {
 
 #if _WITH_TERMIOS
       xiotermios_setflag(pipe->fd, 3, ECHO);
+      xiotermios_flush(pipe->fd);
 #endif /* _WITH_TERMIOS */
       if (pipe->para.readline.prompt || pipe->para.readline.dynprompt) {
 	 /* we must carriage return, because readline will first print the
 	    prompt */
 	 ssize_t writt;
-	 writt = writefull(pipe->fd, "\r", 1);
+	 writt = writefull(pipe->fd, "\r", 1, NULL);
 	 if (writt < 0) {
 	    Warn2("write(%d, \"\\r\", 1): %s",
 		   pipe->fd, strerror(errno));
@@ -199,8 +204,14 @@ ssize_t xioread_readline(struct single *pipe, void *buff, size_t bufsiz) {
       if (line == NULL) {
 	 return 0;	/* EOF */
       }
+#if 0
+      if (strlen(line) == 0) {
+	 Write(STDOUT_FILENO, "\n", 1);
+      }
+#endif
 #if _WITH_TERMIOS
       xiotermios_clrflag(pipe->fd, 3, ECHO);
+      xiotermios_flush(pipe->fd);
 #endif /* _WITH_TERMIOS */
       Add_history(line);
       bytes = strlen(line);
@@ -232,7 +243,7 @@ void xioscan_readline(struct single *pipe, const void *buff, size_t bytes) {
 	 const void *peol = Max(pcr, plf);
 	 /* forget old prompt */
 	 pipe->para.readline.dynend = pipe->para.readline.dynprompt;
-	 len -= (peol+1 - ptr);
+	 len -= ((char *)peol+1 - (char *)ptr);
 	 /* new prompt starts here */
 	 ptr = (const char *)peol+1;
       }
