@@ -11,7 +11,9 @@
 
 /*!! with socat, at most 4 exec children exist */
 pid_t diedunknown[NUMUNKNOWN];	/* children that died before they were registered */
+int   statunknown[NUMUNKNOWN]; 	/* exit state of unknown dead child */
 size_t nextunknown;
+int engine_result = EXIT_SUCCESS;
 
 
 /* register for a xio filedescriptor a callback (handler).
@@ -86,13 +88,15 @@ void childdied(int signum) {
       pid = Waitpid(-1, &status, WNOHANG);
       if (pid == 0) {
 	 Msg(wassig?E_INFO:E_WARN,
-	     "waitpid(-1, {}, WNOHANG): no child has exited");
+	     "waitpid(-1, {}, WNOHANG): no (more) child has exited");
 	 Info("childdied() finished");
 	 diag_in_handler = 0;
 	 errno = _errno;
 	 return;
+      } else if (pid < 0 && errno == EINTR) {
+	 Info1("childdied(): %s", strerror(errno));
       } else if (pid < 0 && errno == ECHILD) {
-	 Msg(wassig?E_INFO:E_WARN,
+	 Msg(wassig?E_INFO:E_NOTICE,
 	      "waitpid(-1, {}, WNOHANG): "F_strerror);
 	 Info("childdied() finished");
 	 diag_in_handler = 0;
@@ -107,22 +111,29 @@ void childdied(int signum) {
 	 errno = _errno;
 	 return;
       }
-   /*! indent */
-   if (num_child) num_child--;
-   /* check if it was a registered child process */
-   i = 0;
-   while (i < XIO_MAXSOCK) {
-      if (xio_checkchild(sock[i], i, pid))  break;
-      ++i;
-   }
-   if (i == XIO_MAXSOCK) {
+
+      /* Check if it was a registered child process
+	 (by EXEC, SHELL, SYSTEM without nofork) */
+      i = 0;
+      while (i < XIO_MAXSOCK) {
+	 if (xio_checkchild(sock[i], i, pid))
+	    break;
+	 ++i;
+      }
+      if (i == XIO_MAXSOCK) {
+	 /* Not found, thus not (yet) registered */
 	 Info2("childdied(%d): cannot identify child %d", signum, pid);
 	 if (nextunknown == NUMUNKNOWN) {
 	    nextunknown = 0;
 	 }
-	 diedunknown[nextunknown++] = pid;
+	 diedunknown[nextunknown] = pid;
+	 statunknown[nextunknown++] = WEXITSTATUS(status);
 	 Debug1("saving pid in diedunknown"F_Zu,
 		nextunknown/*sic, for compatibility*/);
+	 if (num_child) {
+	    --num_child;
+	    Info1("number of children decreased to %d", num_child);
+	 }
       }
 
    if (WIFEXITED(status)) {
@@ -134,8 +145,9 @@ void childdied(int signum) {
 	    Info2("waitpid(): child %d exited with status %d",
 		   pid, WEXITSTATUS(status));
 	 } else {
-	    Error2("waitpid(): child %d exited with status %d",
+	    Warn2("waitpid(): child %d exited with status %d",
 		   pid, WEXITSTATUS(status));
+	    engine_result = 1;
 	 }
       }
    } else if (WIFSIGNALED(status)) {
@@ -143,8 +155,9 @@ void childdied(int signum) {
 	 Info2("waitpid(): child %d exited on signal %d",
 	       pid, WTERMSIG(status));
       } else {
-	 Error2("waitpid(): child %d exited on signal %d",
+	 Warn2("waitpid(): child %d exited on signal %d",
 	       pid, WTERMSIG(status));
+	 engine_result = 1;
       }
    } else if (WIFSTOPPED(status)) {
       Info2("waitpid(): child %d stopped on signal %d",
